/* libwnck header */
/* vim: set sw=2 et: */

/*
 * Copyright (C) 2001 Havoc Pennington
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, see <http://www.gnu.org/licenses/>.
 */

#ifndef WNCK_LIB_H
#define WNCK_LIB_H

#define __LIBWNCK_H_INSIDE__

#include <libwnck/window.h>
#include <libwnck/application.h>
#include <libwnck/class-group.h>
#include <libwnck/workspace.h>
#include <libwnck/screen.h>

#include <libwnck/pager.h>
#include <libwnck/tasklist.h>
#include <libwnck/selector.h>

#include <libwnck/window-action-menu.h>
#include <libwnck/util.h>

#include <libwnck/version.h>

#include <libwnck/wnck-enum-types.h>

#undef __LIBWNCK_H_INSIDE__

#endif /* WNCK_LIB_H */
